Files
TheRock/rocm-libraries/projects/hipthreads/test/hip_thread_condvar_test.cxx
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

38 lines
1.3 KiB
C++

#include "hip/thread"
#include "hip/mutex"
#include "hip/pseudo_mutex" // From hip::wthread library
#include "hip/pseudo_condition_variable"
#include "hip/hip_runtime.h"
#include <cassert>
__device__ void gmain() {
static __device__ hip::pseudo_mutex m1, m2;
static __device__ hip::pseudo_condition_variable cv;
auto other_thread = hip::wthread([&] __device__() {
hip::unique_lock guard1(m1);
hip::unique_lock guard2(m2);
assert(guard1.owns_lock());
assert(guard2.owns_lock());
cv.wait(guard1);
assert(guard1.owns_lock());
assert(guard2.owns_lock());
});
// Keep testing m2 until it's acquired by other_thread, so we know other thread has gotten past the point where it
// locks m1 for the first time.
// Technically the C++ standard doesn't enforce any synchronization between lock and try_lock, but whatever, this isn't a rigorous test anyways
while (m2.try_lock()) {
m2.unlock();
}
hip::unique_lock guard(m1); // wait for other_thread to call cv.wait(guard1)
guard.unlock(); // make sure we unlock m1 so other_thread doesn't spin forever trying to acquire it again when returning from cv.wait
cv.notify_one();
other_thread.join();
}
int main() {
hip::wthread([] __device__(){gmain();}).join();
return 0;
}